dnl ------------------------------ biosapi.S --------------------------------
dnl the original TUNES module to run on raw ISA PCs
dnl
dnl --- see $TUNES/doc/info/BIOS-API.nfo
dnl
dnl
Require({i386})


;;;;;;;;;;;;;;;;;;;;;;;;; Version identification ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
define({API_MAGIC},{MAGIC_PC_API})
define({API_NAME},{"OSLm i386 BIOS API"})


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Constants ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
define({APISEG},	0x0000)		; Where the API is originally loaded
define({APIOFS},	PAGESIZE)	; The corresponding offset in PM.
define({IRQ_INT_BASE},	0x90)		; IRQ's are mapped at INT 90h
define({IDT_SIZE},	0xA0*8)		; Is that enough ?
define({GDT_SIZE},	0x1000)		; Is that enough ?
define({COMMAND_LINE},	0x800)		; Where to put the command line
define({V86_SERVER_INT},0xFD)		; V86 server interrupt
define({PM_STACK_SIZE},	0x4000)		; 4096 entries

ASCII_defs
Std_Struc
API_Struc

;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Some options ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
define({__IO_Delay__},
	{locals({D1},{D2},{jmp D1}__nl__{D1:	jmp D2}__nl__{D2:})})dnl


;;;;;;;;;;;;;;;;;;;;;;;;; Standard PM Selectors ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Name			sel.	base lim.	description

;; These are valid even when paging is disabled
;; thus, linear memory must be identity-mapped to physical memory,
;; or at least the PMPART of the API

NULLSEG		=	0x0000	; Invalid null segment
TEMPSEG		=	0x0008	; Temporary
ALLMEMCS	=	0x0010	; Whole memory (code), both
ALLMEMDS	=	0x0018	; Whole memory (data), both
RMPARTCS	=	0x0020	; RM Part of the API (code), noPG/IdPG
RMPARTDS	=	0x0028	; RM Part of the API (data), noPG/IdPG
PMPARTCS	=	0x0030	; PM Part of the API (code), noPG/IdPG
PMPARTDS	=	0x0038	; PM Part of the API (data), noPG/IdPG

KL0CS		=	0x0040	; Kernel PL0 Code Segment, both w/ trick
KL0DS		=	0x0048	; Kernel PL0 Data Segment, both w/ trick
KL1CS		=	0x0051	; Kernel PL1 Code Segment, PG
KL1DS		=	0x0059	; Kernel PL1 Data Segment, PG
KL2CS		=	0x0062	; Kernel PL2 Code Segment, PG
KL2DS		=	0x006A	; Kernel PL2 Data Segment, PG
KL3CS		=	0x0073	; Kernel PL3 Code Segment, PG
KL3DS		=	0x007B	; Kernel PL3 Data Segment, PG

KL0CG		=	0x0080	; Kernel PL0 Call Gate
KL1CG		=	0x0089	; Kernel PL1 Call Gate
KL2CG		=	0x0092	; Kernel PL2 Call Gate
KL3CG		=	0x009B	; Kernel PL3 Call Gate

KTSS		=	0x00A0	; Kernel TSS, PG
DummyTSS	=	0x00A8	; dummy for non-saving "switches"
V86TSS		=	0x00B0	; V86 TSS, PG
UserTSS		=	0x00B8	; User TSS, PG

; What about adding more tasks, or task gates ?

define({Orig_GDT_SIZE},0x00C0*8)

SysCallArgC	=	8	; Number of arguments to a "system call"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; segmenting ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; .text is rmpart
; .data is pmpart
; .bss is pmpart udata.   Could anyone tell me what "bss" stands for ??????

; after all, everything is .text, so as86 is not confused

declarediversion({__rminitcode__})dnl
declarediversion({__rminitdata__})dnl
declarediversion({__rmcode__})dnl
declarediversion({__rmstringseg__})dnl
declarediversion({__rmdata__})dnl
declarediversion({__rmudata__})dnl
declarediversion({__pminitcode__})dnl
declarediversion({__pminitdata__})dnl
declarediversion({__pmcode__})dnl
declarediversion({__pmstringseg__})dnl
declarediversion({__symbolseg__})dnl
declarediversion({__pmdata__})dnl
declarediversion({__pmdataend__})dnl
declarediversion({__pmudata__})dnl
declarediversion({__pmudataend__})dnl


define({pmaddr},	{pmofs+$1})
define({rmaddr},	{rmofs+$1})
define({initaddr},	{$1})


;;;;;;;;;;;;;;;;;;;;;;;;;; Initialize Segments ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

define({__begin__},{
__pmcode__({
;;; SEGMENT: PM CODE
.align 16
use32
pmcode_beg:
})
__pmdata__({
;;; SEGMENT: PM DATA
.align 16
pmdata_beg:
dnl .text
})
__pmudata__({
;;; SEGMENT: PM UDATA
pmudata_beg:
newheap({u},pmudata_beg,0)
})
__pminitcode__({
;;; SEGMENT: PM INIT CODE
.align 16
use32
pminitcode_beg:
})
__pminitdata__({
;;; SEGMENT: PM INIT DATA
.align 16
pminitdata_beg:
})
__rmcode__({
;;; SEGMENT: RM CODE
dnl .text
use16
.align 16
rmcode_beg:
})
__rmdata__({
;;; SEGMENT: RM DATA
.align 16
rmdata_beg:
})
__rmudata__({
;;; SEGMENT: RM UDATA
dnl .text
.align 16
rmudata_beg:
newheap({ru},rmudata_beg,0)
})
__rminitcode__({
;;; SEGMENT: RM INIT CODE
.align 16
use16
rminitcode_beg:
})
__rminitdata__({
;;; SEGMENT: RM INIT DATA
.align 16
dnl .text
rminitdata_beg:
})
__rmstringseg__({
;;; SEGMENT: RM STRING
.align 16
rmstringseg_beg:
})
__pmstringseg__({
;;; SEGMENT: PM STRING
dnl .text
.align 16
pmstringseg_beg:
})
__symbolseg__({
;;; SEGMENT: SYMBOL
.align 16
symbolseg_beg:

})})

define({rminit},{
	define({__codeseg__},{__rminitcode__($}{@)})
	define({__dataseg__},{__rminitdata__($}{@)})
	define({__udataseg__},{__rminitdata__($}{@)})
	define({__stringseg__},{__rminitdata__($}{@)})
	define({codeaddr},{rmaddr($}{1)})
	define({stringaddr},{rmaddr($}{1)})})
define({pminit},{
	define({__codeseg__},{__pminitcode__($}{@)})
	define({__dataseg__},{__pminitdata__($}{@)})
	define({__udataseg__},{__pminitdata__($}{@)})
	define({__stringseg__},{__pminitdata__($}{@)})
	define({codeaddr},{pmaddr($}{1)})
	define({stringaddr},{pmaddr($}{1)})})
define({pmpart},{
	define({__codeseg__},{__pmcode__($}{@)})
	define({__dataseg__},{__pmdata__($}{@)})
	define({__udataseg__},{__pmudata__($}{@)})
	define({__stringseg__},{__pmstringseg__($}{@)})
	define({codeaddr},{pmaddr($}{1)})
	define({stringaddr},{pmaddr($}{1)})})
define({rmpart},{
	define({__codeseg__},{__rmcode__($}{@)})
	define({__dataseg__},{__rmdata__($}{@)})
	define({__udataseg__},{__rmudata__($}{@)})
	define({__stringseg__},{__rmstringseg__($}{@)})
	define({codeaddr},{rmaddr($}{1)})
	define({stringaddr},{rmaddr($}{1)})})


define({__end__},{

__pmcode__(
{
.align 16
pmcode_end:
})
__pmdata__(
{
.align 16
})
__pmdataend__(
{
.align 16
pmdata_end:
})
__pmudata__(
{
alignheap(u,16)
})
__pmudataend__(
{
pmudata_end = alallot(u,16,0)
})
__pminitcode__(
{
.align 16
pminitcode_end:
})
__pminitdata__(
{
.align 16
pminitdata_end:
})
__rmcode__(
{
.align 16
rmcode_end:
})
__rmdata__(
{
.align 16
rmdata_end:
})
__rmudata__(
{
rmudata_end = alallot(ru,16,0)
})
__rminitcode__(
{
.align 16
rminitcode_end:
})
__rminitdata__(
{
.align 16
rminitdata_end:
})
__rmstringseg__(
{
.align 16
rmstringseg_end:
})
__pmstringseg__(
{
.align 16
pmstringseg_end:
})
__symbolseg__(
{
.align 16
symbolseg_end:
})
;undiverting
undivert


})


; ---------------- Some constants -----------------

rminitcode_len	= rminitcode_end - rminitcode_beg
rminitdata_len	= rminitdata_end - rminitdata_beg
rmcode_len	= rmcode_end - rmcode_beg
rmdata_len	= rmdata_end - rmdata_beg
rmudata_len	= rmudata_end - rmudata_beg
rmstringseg_len	= rmstringseg_end - rmstringseg_beg

pminitcode_len	= pminitcode_end - pminitcode_beg
pminitdata_len	= pminitdata_end - pminitdata_beg
pmcode_len	= pmcode_end - pmcode_beg
pmdata_len	= pmdata_end - pmdata_beg
pmudata_len	= pmudata_end - pmudata_beg
pmstringseg_len	= pmstringseg_end - pmstringseg_beg
symbolseg_len	= symbolseg_end - symbolseg_beg

pmpart_beg	= pmcode_beg
pmpart_end	= pmudata_end
pmpart_len	= pmpart_end - pmpart_beg
pmpart_actbeg	= pminitcode_beg
pmpart_actend	= pmdata_end
pmpart_actlen	= pmpart_actend - pmpart_actbeg
pmpart_bsslen	= pmudata_len

rmpart_beg	= rmcode_beg
rmpart_end	= rmudata_end
rmpart_len	= rmpart_end - rmpart_beg
rmpart_actbeg	= rminitcode_beg
rmpart_actend	= rmdata_end
rmpart_actlen	= rmpart_actend - rmpart_actbeg
rmpart_bsslen	= rmudata_len

total_length	= rmpart_actlen + pmpart_actlen

pmofs	= -pmpart_end
rmofs	= 0x10000-rmpart_end

APISIZE = total_length



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Actual code ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
define({DEBUG})
define({__DEBUG__})


.org APIOFS

_VERBOSEln_({beginning})
__begin__

zzz({
Require({debug})
Require({rmdebug})
})

Require({header})
Require({head})
Require({rmpart})
Require({pmpart})
Require({xrv86})

Require({video})
;;Require({vp})

Require({paging})	; (!) must be last in list


_VERBOSEln_({end reached})
__end__
